#!/usr/bin/ruby

require 'optparse'
require 'nodesets'

options = {
  :field_separator  => ' ',
}

optparse = OptionParser.new do|opts|
  opts.banner = "Usage: nodeset.rb [options]"
  opts.on( '--debug', 'Support pry for debugging' ) do
    options[:debug] = true
  end
  opts.on( '-v', '--verbose', 'Output a lot of job information' ) do
    options[:verbose] = true
  end
  opts.on( '-f', '--fs \'separator...\'', 'Field separator to use between nodes, defaults to space' ) do |fs|
    options[:field_separator] = fs.gsub(/(#{SEPARATORS.keys.map{|x|Regexp.escape(x)}.join('|')})/,SEPARATORS)
  end
  opts.on( '-e', '--expand [NodeSet]', 'Expand NodeSet to a list' ) do |nodes|
    options[:expand] = nodes ? nodes : ''
  end
  opts.on( '-f', '--fold [Nodes]', 'Fold a node list of most forms into a proper NodeSet' ) do |nodes|
    options[:fold] = nodes ? nodes : ''
  end
  opts.on( '-c', '--count [Nodes]', 'Count a node list of most forms of NodeSet' ) do |nodes|
    options[:count] = nodes ? nodes : ''
  end
  opts.on( '-x', '--exclude Nodes', 'Exclude specified NodeSet from inputted NodeSet' ) do |nodes|
    options[:exclude] = nodes
  end
  opts.on( '-i', '--intersection Nodes', 'Calculate specified NodeSet intersection from inputted NodeSet' ) do |nodes|
    options[:intersection] = nodes
  end
  opts.on( '-X', '--xor Nodes', 'Calculate symmetric difference between this NodeSet and inputted NodeSet' ) do |nodes|
    options[:xor] = nodes
  end
end
begin optparse.parse!
  rescue OptionParser::InvalidOption => e
  puts e
  puts optparse
  exit 1
end

if ARGF.filename != "-" or (not STDIN.tty? and not STDIN.closed?)
  input = ARGF.read.gsub("\n", ',').chomp(',')
end

fail 'Can only set either --fold or --expand, not both' if options[:fold] && options[:expand]
fail 'Can only set one or none of --exclude, --intersection or --xor' if [options[:exclude],options[:intersection],options[:xor]].compact.count > 1

nodeset = Nodesets::Nodeset.new
[options[:expand], options[:fold], options[:count], input].each do |nodes|
  nodeset.merge(nodes) if not nodes.nil? || nodes.empty?
end

out = []
to_out = []
if options[:exclude]
  nodeset = nodeset - options[:exclude]
elsif options[:intersection]
  nodeset = nodeset & options[:intersection]
elsif options[:xor]
  nodeset = nodeset ^ options[:xor]
end
out << nodeset.count if options[:count]
if options[:expand]
  out.concat(nodeset.to_a)
elsif options[:fold]
  out << nodeset.fold
end
to_out << out.join(options[:field_separator]) if out
puts to_out
