#!/usr/bin/ruby

require 'optparse'
require 'nodesets'

options = {
  :field_separator  => ' ',
}

optparse = OptionParser.new do|opts|
  opts.banner = "Usage: nodeset_oo_example.rb [options]"
  opts.on( '--debug', 'Support pry for debugging' ) do
    options[:debug] = true
  end
  opts.on( '-v', '--verbose', 'Output a lot of job information' ) do
    options[:verbose] = true
  end
  opts.on( '-f', '--fs \'separator...\'', 'Field separator to use between nodes, defaults to space' ) do |fs|
    options[:field_separator] = fs.gsub(/(#{SEPARATORS.keys.map{|x|Regexp.escape(x)}.join('|')})/,SEPARATORS)
  end
  opts.on( '-e', '--expand [NodeSet]', 'Expand NodeSet to a list' ) do |nodes|
    options[:expand] = nodes ? nodes : ''
  end
  opts.on( '-f', '--fold [Nodes]', 'Fold a node list of most forms into a proper NodeSet' ) do |nodes|
    options[:fold] = nodes ? nodes : ''
  end
  opts.on( '-c', '--count [Nodes]', 'Count a node list of most forms of NodeSet' ) do |nodes|
    options[:count] = nodes ? nodes : ''
  end
end
begin optparse.parse!
  rescue OptionParser::InvalidOption => e
  puts e
  puts optparse
  exit 1
end

if ARGF.filename != "-" or (not STDIN.tty? and not STDIN.closed?)
  input = ARGF.read
end

fail 'Can only set either --fold or --expand, not both' if options[:fold] && options[:expand]

node_out = []
to_out = []
expanded = nil
expanded_of_folded = nil
if not options[:expand].nil?
  options[:expand] = options[:expand]+','+input if input
  expanded = Nodesets.parse_nodeset(options[:expand])
  node_out << expanded
elsif not options[:fold].nil?
  options[:fold] = options[:fold]+','+input if input
  expanded_of_folded = Nodesets.parse_nodeset(options[:fold])
  node_out << [Nodesets.make_nodeset(expanded_of_folded)]
end
if not options[:count].nil?
  incl_input = nil
  incl_count = nil
  incl_count = count if not options[:count].empty?
  incl_input = input if not options[:expand] && options[:fold]
  nodes = [expanded, expanded_of_folded, incl_count, incl_input].compact.join(',')
  to_out << Nodesets.parse_nodeset(nodes).count
end
to_out << node_out.join(options[:field_separator]) if node_out
puts to_out.flatten
